@@ -1,5 +1,5 @@
-#ifndef INCLUDED_APPEND_LIST_H
-#define INCLUDED_APPEND_LIST_H
+#ifndef APPEND_LIST_H
+#define APPEND_LIST_H
 
 #include "struct_VList.h"
 #include "xmalloc.h" // inline
