@@ -90,9 +90,11 @@ static off_t total_read_local;
 static off_t total_write_local;
 static off_t total_read_error; // スレッド用
 static off_t total_write_error; // スレッド用
-static off_t buffer_size_local;
 static int ifrom;
 static int ito;
+static int buffer_size;
+static int buffer_size_half;
+static int buffer_size_thread;
 static int rbuf_num;
 static int wbuf_num;
 static long psize;
@@ -131,12 +133,21 @@ void read_write(const OPArg *oparg, CPInfo *cpinfo, SDir *sdir, CPDD *cpdd)
 	total_write_local = 0;
 	total_read_error = 0;
 	total_write_error = 0;
-	buffer_size_local = 0;
 	ifrom = 0;
 	ito = 0;
 	rbuf_num = 0;
 	wbuf_num = 0;
 
+	if(buffer_size == 0)
+	{
+		buffer_size = oparg->buffer_size;
+	}
+
+	if(buffer_size_half == 0)
+	{
+		buffer_size_half = buffer_size / 2;
+	}
+
 	if(psize == 0)
 	{
 		psize = sysconf(_SC_PAGESIZE);
@@ -320,12 +331,17 @@ void read_write(const OPArg *oparg, CPInfo *cpinfo, SDir *sdir, CPDD *cpdd)
 			printf("%s -> %s\n", cpinfo->from, cpinfo->to);
 		}
 
-		if(oparg->buffer_size >= from_size_local)
+		// 1048576 == 1024 * 1024 == 1MB
+		if(1048576 > from_size_local)
 		{
 			errno = 0;
 			buf[0] = mmap(NULL, from_size_local, PROT_READ, MAP_PRIVATE, ifrom, 0);
 
-			if(buf[0] != MAP_FAILED)
+			if(buf[0] == MAP_FAILED)
+			{
+				perror("mmap");
+			}
+			else
 			{
 				total_read_local = from_size_local;
 
@@ -379,14 +395,43 @@ void read_write(const OPArg *oparg, CPInfo *cpinfo, SDir *sdir, CPDD *cpdd)
 					print_error("munmap", __FILE__, __LINE__, cpdd);
 				}
 			}
-			else
-			{
-				perror("mmap");
-			}
 		}
 		else if(cpinfo->copy_mode == DIFFERENT)
 		{
-			buffer_size_local = oparg->buffer_size_thread;
+			{
+				VList *b_size_list = oparg->b_size_list;
+				b_size_list = b_size_list->tail;
+
+				int *i = (int *)b_size_list->data;
+				int low = *i;
+
+				b_size_list = b_size_list->prev;
+				int high;
+
+				for(;;)
+				{
+					i = (int *)b_size_list->data;
+					high = *i;
+
+					if(high > from_size_local)
+					{
+						buffer_size_thread = low;
+						break;
+					}
+
+					if(b_size_list->prev == NULL)
+					{
+						buffer_size_thread = buffer_size_half;
+						break;
+					}
+					else
+					{
+						low = high;
+						b_size_list = b_size_list->prev;
+					}
+				}
+			}
+
 			rbuf_num = 0;
 
 			pthread_t t1;
@@ -472,9 +517,9 @@ void read_write(const OPArg *oparg, CPInfo *cpinfo, SDir *sdir, CPDD *cpdd)
 			{
 				off_t read_size = from_size_local - total_read_local;
 
-				if(read_size >= oparg->buffer_size)
+				if(read_size >= buffer_size)
 				{
-					read_size = oparg->buffer_size;
+					read_size = buffer_size;
 				}
 				else
 				{
@@ -489,7 +534,12 @@ void read_write(const OPArg *oparg, CPInfo *cpinfo, SDir *sdir, CPDD *cpdd)
 				errno = 0;
 				buf[0] = mmap(NULL, read_size, PROT_READ, MAP_PRIVATE, ifrom, total_read_local);
 
-				if(buf[0] != MAP_FAILED)
+				if(buf[0] == MAP_FAILED)
+				{
+					perror("mmap");
+					break;
+				}
+				else
 				{
 					total_read_local += read_size;
 
@@ -541,11 +591,6 @@ void read_write(const OPArg *oparg, CPInfo *cpinfo, SDir *sdir, CPDD *cpdd)
 						print_error("munmap", __FILE__, __LINE__, cpdd);
 					}
 				}
-				else
-				{
-					perror("mmap");
-					break;
-				}
 
 				if(total_read_local >= from_size_local)
 				{
@@ -732,9 +777,9 @@ static void * read_thread(void *unknown)
 
 	off_t read_size = from_size_local - total_read_local;
 
-	if(read_size >= buffer_size_local)
+	if(read_size >= buffer_size_thread)
 	{
-		read_size = buffer_size_local;
+		read_size = buffer_size_thread;
 	}
 	else
 	{
@@ -747,7 +792,18 @@ static void * read_thread(void *unknown)
 
 	errno = 0;
 	buf[i] = mmap(NULL, read_size, PROT_READ, MAP_PRIVATE, ifrom, total_read_local);
-	if(buf[i] != MAP_FAILED)
+
+	if(buf[i] == MAP_FAILED)
+	{
+		// このcpddに意味はない。print_errorの引数用。
+		static CPDD cpdd;
+		cpdd.total_error = 0;
+		print_error("mmap", __FILE__, __LINE__, &cpdd);
+		total_read_error++;
+
+		next_write_size[i] = 0;
+	}
+	else
 	{
 		errno = 0;
 		// マルチスレッドの場合、
@@ -764,10 +820,6 @@ static void * read_thread(void *unknown)
 		total_read_local += read_size;
 		next_write_size[i] = read_size;
 	}
-	else
-	{
-		next_write_size[i] = 0;
-	}
 
 	return NULL;
 }
